 /********************************************************************************************
 *LOGO-EM v0.19
 *
 *Die Demo-Firmware LOGO-EM ([E]rweiterungs[M]odul) für die Hardware AKtron-IO ermöglicht in
 *Verbindung mit dem Protokollkonverter InLOGO das Erweitern auf bis zu 64 digitale Ein- und 
 *64 Digitale Ausgänge an einem LOGO!8 Basismodul.
 *Weil AKtron-IO mehr Ein-/Ausgänge als das Arduino Mega2560 Board hat, wird auf
 *digitalRead() und digitalWrite() verzichtet und stattdessen Port-/Byteweise gelesen
 *und geschrieben.
 *Fuer das LOGO!8 Basismodul steht ein entsprechendes LOGO!Soft Comfort (LSC) Schaltprogramm
 *zur Verfügung.
 *
 *IDE:  Arduino 1.8.x
 *Mikrocontroller:  ATmega2560
 *Boardauswahl : Arduino AVR Boards -> Mega 2560
 ********************************************************************************************/
/********************************************************************************************
 * Die hier unter GNU General Public Lizenz v3 kostenlose und zur freien Verfuegung gestellte
 * OSS (Open Source Software) hat den Zweck dem Nutzer den Einstieg zu erleichtern und kann 
 * als Grundlage für eigene Erweiterungen, Projekte oder Schulungszwecke verwendet werden.
 * Bei den Softwareversionen 0.x handelt es sich um Demo-Versionen bei denen für Demozwecke
 * von „ausreichendem“ Funktionsumfang auszugehen ist.
 * Es besteht KEIN rechtlicher Anspruch auf Vollständigkeit und Fehlerfreiheit.
 *
 * Copyleft Andreas Kaestner https://aktron.de
 ********************************************************************************************/

// BOF preprocessor bug prevent - insert me on top of your arduino-code
#if 1
__asm volatile ("nop");
#endif

#define  EE_MySlaveID   152
#define  EE_Twi         153  // Twi (I2C) [0...1]
#define  EE_LinkMode    154  // LinkMode [0...2] 
                             // [0] = TTL über Serial3
                             // [1] = RS485 über Serial1
                             // [2] = 3DR Radio Telemetrie über Serial3
#define LED_ON  PORTJ|=128;
#define LED_OFF PORTJ&=127;

#include <avr/wdt.h>
#include <EEPROM.h>
#include "Adafruit_GFX.h"
#include "SH1106.h"

#define OLED_RESET 4
SH1106 display(OLED_RESET);

#define XPOS 0
#define YPOS 1

#if (SH1106_LCDHEIGHT != 64)
 #error("Height incorrect, please fix SH1106.h!");
#endif

String  sVersion = " v0.19";

// Array für Rx-Kommunikationsausfall
bool RxComActive[4];

unsigned long actComTime[4]; // aktuelle Zeit
unsigned long preComTime[4]; // vorherige Zeit


// Variablen für Rx-Byte Empfangsende
unsigned long actRxTime[4]; // aktuelle Zeit
unsigned long preRxTime[4]; // vorherige Zeit

bool RxTimeOver[4] = {true, true, true, true};

unsigned long actGetIOTime, preGetIOTime;
unsigned long actTxDelay[4], preTxDelay[4];
boolean       xStartTxTimer[4] = {false, false, false, false};
byte          bComFailureTime;
byte          bDifTxDelay = 0;


// 1. Eintrag ist die komplette Array-Länge
// Aufbau RxBefehl von InLogo!
/*
00 - 11 Länge
01 - von ID
02 - nach ID
03 - 0x11 Kommando-Typ
04 - Byte 0 Ausgang
05 - Byte 1 Ausgang
06 - Byte 2 Ausgang
07 - Byte 3 Ausgang
08 - 0x3A Ende
09 - Prüfsumme lowbyte (von Länge ... 0x3A
10 - Prüfsumme highbyte
*/

byte RxBuf[4][16];
byte RxCnt[4]  = {0, 0, 0, 0};

byte bSendComModeToMaster [4] = {4, 1, 0, 0xA0}; 


// Aufbau TxBefehl zu InLOGO!
/*
00 - 11 Länge
01 - von ID
02 - nach ID
03 - 0x10 Kommando-Typ
04 - Byte 0 Eingang
05 - Byte 1 Eingang
06 - Byte 2 Eingang
07 - Byte 3 Eingang
08 - 0x3A Ende
09 - Prüfsumme lowbyte (von Länge ... 0x3A
10 - Prüfsumme highbyte
*/


unsigned int iLoop = 0;
char c[64];
String str;

byte    b = 0;
bool    x = false;
byte    bBoardType = 0;
String  sCmd;
byte    bMySlaveID;
boolean xTwi = true;
byte bLinkMode  = 0;


// Das deklarierte Tx-Array komplett definieren mit 0xFF
byte TxBuf[4][16] = {11, bMySlaveID, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x3A, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                     11, bMySlaveID, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x3A, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                     11, bMySlaveID, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x3A, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
                     11, bMySlaveID, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x3A, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
byte *pTx[4];

byte bP, bX;          //  bP-Port 0...7, bX-Bit 0...7

// Ausgangs-Ports 4...7
byte    bQ[8]             = {0,0,0,0,0,0,0,0};
byte    bQPre[8]          = {0,0,0,0,0,0,0,0};  // vorheriger Zustand Eingang
byte    bQR[8]            = {0,0,0,0,0,0,0,0};  // Flanke positiv (rise)
byte    bQF[8]            = {0,0,0,0,0,0,0,0};  // Flanke negativ (fall)
byte    bQC[8]            = {0,0,0,0,0,0,0,0};  // PinChange

// Eingangs-Ports 0...3
byte    bI[8]             = {0,0,0,0,0,0,0,0};  // aktueller Zustand Eingang
byte    bIPre[8]          = {0,0,0,0,0,0,0,0};  // vorheriger Zustand Eingang
byte    bIR[8]            = {0,0,0,0,0,0,0,0};  // Flanke positiv (rise)
byte    bIF[8]            = {0,0,0,0,0,0,0,0};  // Flanke negativ (fall)
byte    bIC[8]            = {0,0,0,0,0,0,0,0};  // PinChange

byte    bReverse[8]       = {0,0,0,0,0,0,0,0};  // Port-Pins revers behandeln ( 0 = normal, 1 = revers bzw. gespiegelt)
byte    bInverse[8]       = {0,0,0,0,0,0,0,0};  // Port invers behandeln ( 0 = normal, 1 = invertiert)

// ************************************************************* 
//   ReverseByte() Eingangs Byte wird spielgelverkehrt
//   zurueck gegeben.
// *************************************************************
byte ReverseByte(byte in)
{
  byte out = 0;
  if (in & 0x01) out |= 0x80;
  if (in & 0x02) out |= 0x40;
  if (in & 0x04) out |= 0x20;
  if (in & 0x08) out |= 0x10;
  if (in & 0x10) out |= 0x08;
  if (in & 0x20) out |= 0x04;
  if (in & 0x40) out |= 0x02;
  if (in & 0x80) out |= 0x01;

  return out;
}

 

 
// ************************************************************* 
// Wird nach Kommunikationsunterbrechung zu InLOGO aufgerufen
// *************************************************************
 void ReIni()
{
 RxCnt[0] =   RxCnt[1] = 0;
 RxBuf[0][0]=  RxBuf[1][0] = 0;
}
