/*********************************************************************
This is a library for our Monochrome OLEDs based on SH1106 drivers

  Pick one up today in the adafruit shop!
  ------> http://www.adafruit.com/category/63_98

These displays use SPI to communicate, 4 or 5 pins are required to  
interface

Adafruit invests time and resources providing this open source code, 
please support Adafruit and open-source hardware by purchasing 
products from Adafruit!

Written by Limor Fried/Ladyada  for Adafruit Industries.  
BSD license, check license.txt for more information
All text above, and the splash screen must be included in any redistribution
*********************************************************************/


 #include "Arduino.h"


  typedef volatile uint8_t PortReg;
  typedef uint8_t PortMask;



#include "Adafruit_GFX.h"

#define BLACK 0
#define WHITE 1
#define INVERSE 2

#define swap(a, b) { int16_t t = a; a = b; b = t; }

#define SH1106_I2C_ADDRESS   0x3C	// 011110+SA0+RW - 0x3C or 0x3D
// Address for 128x32 is 0x3C
// Address for 128x64 is 0x3D (default) or 0x3C (if SA0 is grounded)

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif

#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#define TWI_FREQ 400000

  #define SH1106_LCDWIDTH                  128
  #define SH1106_LCDHEIGHT                 64


#define SH1106_SETCONTRAST 0x81
#define SH1106_DISPLAYALLON_RESUME 0xA4
#define SH1106_DISPLAYALLON 0xA5
#define SH1106_NORMALDISPLAY 0xA6
#define SH1106_INVERTDISPLAY 0xA7
#define SH1106_DISPLAYOFF 0xAE
#define SH1106_DISPLAYON 0xAF

#define SH1106_SETDISPLAYOFFSET 0xD3
#define SH1106_SETCOMPINS 0xDA

#define SH1106_SETVCOMDETECT 0xDB

#define SH1106_SETDISPLAYCLOCKDIV 0xD5
#define SH1106_SETPRECHARGE 0xD9

#define SH1106_SETMULTIPLEX 0xA8

#define SH1106_SETLOWCOLUMN 0x00
#define SH1106_SETHIGHCOLUMN 0x10

#define SH1106_SETSTARTLINE 0x40

#define SH1106_MEMORYMODE 0x20
#define SH1106_COLUMNADDR 0x21
#define SH1106_PAGEADDR   0x22

#define SH1106_COMSCANINC 0xC0
#define SH1106_COMSCANDEC 0xC8

#define SH1106_SEGREMAP 0xA0

#define SH1106_CHARGEPUMP 0x8D

#define SH1106_EXTERNALVCC 0x1
#define SH1106_SWITCHCAPVCC 0x2

#define SH1106_SETSTARTPAGE 0xB0

// Scrolling #defines
#define SH1106_ACTIVATE_SCROLL 0x2F
#define SH1106_DEACTIVATE_SCROLL 0x2E
#define SH1106_SET_VERTICAL_SCROLL_AREA 0xA3
#define SH1106_RIGHT_HORIZONTAL_SCROLL 0x26
#define SH1106_LEFT_HORIZONTAL_SCROLL 0x27
#define SH1106_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL 0x29
#define SH1106_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL 0x2A

class SH1106 : public Adafruit_GFX {
 public:

 byte bCommandTwi    = 1;
 byte bStartSimultan = 1;
 byte WriteToDisplayLoop = 0;
 byte pixel = 0;
 byte xData = 0;
 byte commandStep = 1; 
 byte pixelStep = 1;
 byte dataStep = 1;

 boolean xTwiBegin = false;
 boolean xIsTwiBeginRunning = false;
 byte bTwiBeginStep = 1;
 
// uint8_t buffer[SH1106_LCDHEIGHT * SH1106_LCDWIDTH / 8];
  SH1106(int8_t SID, int8_t SCLK, int8_t DC, int8_t RST, int8_t CS);
  SH1106(int8_t DC, int8_t RST, int8_t CS);
  SH1106(int8_t RST);

  uint8_t buffer[SH1106_LCDHEIGHT * SH1106_LCDWIDTH / 8];
  void ClearRegion(uint16_t from, uint16_t to);
  void TwiBeginSimultan();
  void ReIniDisplay();

  void begin();
  void SH1106_command(uint8_t c);
  void SH1106_data(uint8_t c);

  uint8_t SH1106_commandTwi(uint8_t c);
  byte _i2caddr;

  void clearDisplay(void);
  void invertDisplay(uint8_t i);
  void display();
  void WriteToDisplay(void);

  void TWIInit(void);
  void TWIStart(uint8_t bAdr);
  void TWIStop(void);
  void TWIWrite(uint8_t data);

  uint8_t TWIStartSimultan(uint8_t bAdr);
  uint8_t TWIWriteSimultan(uint8_t data);
  
  void startscrollright(uint8_t start, uint8_t stop);
  void startscrollleft(uint8_t start, uint8_t stop);

  void startscrolldiagright(uint8_t start, uint8_t stop);
  void startscrolldiagleft(uint8_t start, uint8_t stop);
  void stopscroll(void);

  void dim(boolean dim);

  void drawPixel(int16_t x, int16_t y, uint16_t color);

  virtual void drawFastVLine(int16_t x, int16_t y, int16_t h, uint16_t color);
  virtual void drawFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color);

 private:
  int8_t  _vccstate, sid, sclk, dc, rst, cs;
  void fastSPIwrite(uint8_t c);

  boolean hwSPI;
  PortReg *mosiport, *clkport, *csport, *dcport;
  PortMask mosipinmask, clkpinmask, cspinmask, dcpinmask;

  inline void drawFastVLineInternal(int16_t x, int16_t y, int16_t h, uint16_t color) __attribute__((always_inline));
  inline void drawFastHLineInternal(int16_t x, int16_t y, int16_t w, uint16_t color) __attribute__((always_inline));

};
